import { z } from "https://deno.land/x/zod@v3.22.4/mod.ts";

export type Rule = {
	path: string;
	kind: "include" | "exclude";
};

export class ModuleLayout {
	rules;

	constructor(rules: Rule[]) {
		this.rules = rules;
	}

	public static new_empty() {
		return new ModuleLayout([]);
	}

	public append_rule(rule: Rule) {
		//
	}

	public package_to() {
	}
}

export type ModuleFile = {
	path: string;
	kind: "text" | "bin";
	content: string;
};

export class Module {
	layout;
	content;

	constructor(layout: ModuleLayout, content: ModuleFile[]) {
		this.layout = layout;
		this.content = content;
	}

	public serialize() {
		const layout = { rules: this.layout.rules };
		const content = this.content;
		JSON.stringify({ layout, content });
	}

	public static deserialize(serialized: string) {
		const { layout: { rules }, content } = z.object({
			layout: z.object({
				rules: z.array(z.object({
					path: z.string(),
					kind: z.literal("include").or(z.literal("exclude")),
				})),
			}),
			content: z.array(z.object({
				path: z.string(),
				kind: z.literal("text").or(z.literal("bin")),
				content: z.string(),
			})),
		}).parse(JSON.parse(serialized));
		const layout = new ModuleLayout(rules);
		return new Module(layout, content);
	}
}
